# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Dict, Mapping, cast
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import CasedevError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        llm,
        ocr,
        agent,
        legal,
        vault,
        voice,
        format,
        memory,
        search,
        system,
        compute,
        database,
        superdoc,
        privilege,
        translate,
        applications,
    )
    from .resources.system import SystemResource, AsyncSystemResource
    from .resources.llm.llm import LlmResource, AsyncLlmResource
    from .resources.ocr.ocr import OcrResource, AsyncOcrResource
    from .resources.agent.agent import AgentResource, AsyncAgentResource
    from .resources.legal.legal import LegalResource, AsyncLegalResource
    from .resources.vault.vault import VaultResource, AsyncVaultResource
    from .resources.voice.voice import VoiceResource, AsyncVoiceResource
    from .resources.format.format import FormatResource, AsyncFormatResource
    from .resources.memory.memory import MemoryResource, AsyncMemoryResource
    from .resources.search.search import SearchResource, AsyncSearchResource
    from .resources.compute.compute import ComputeResource, AsyncComputeResource
    from .resources.database.database import DatabaseResource, AsyncDatabaseResource
    from .resources.superdoc.superdoc import SuperdocResource, AsyncSuperdocResource
    from .resources.privilege.privilege import PrivilegeResource, AsyncPrivilegeResource
    from .resources.translate.translate import TranslateResource, AsyncTranslateResource
    from .resources.applications.applications import ApplicationsResource, AsyncApplicationsResource

__all__ = [
    "ENVIRONMENTS",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Casedev",
    "AsyncCasedev",
    "Client",
    "AsyncClient",
]

ENVIRONMENTS: Dict[str, str] = {
    "production": "https://api.case.dev",
    "local": "http://localhost:2728",
}


class Casedev(SyncAPIClient):
    # client options
    api_key: str

    _environment: Literal["production", "local"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        environment: Literal["production", "local"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Casedev client instance.

        This automatically infers the `api_key` argument from the `CASEDEV_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("CASEDEV_API_KEY")
        if api_key is None:
            raise CasedevError(
                "The api_key client option must be set either by passing api_key to the client or by setting the CASEDEV_API_KEY environment variable"
            )
        self.api_key = api_key

        self._environment = environment

        base_url_env = os.environ.get("CASEDEV_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `CASEDEV_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def agent(self) -> AgentResource:
        from .resources.agent import AgentResource

        return AgentResource(self)

    @cached_property
    def system(self) -> SystemResource:
        from .resources.system import SystemResource

        return SystemResource(self)

    @cached_property
    def applications(self) -> ApplicationsResource:
        from .resources.applications import ApplicationsResource

        return ApplicationsResource(self)

    @cached_property
    def compute(self) -> ComputeResource:
        from .resources.compute import ComputeResource

        return ComputeResource(self)

    @cached_property
    def database(self) -> DatabaseResource:
        from .resources.database import DatabaseResource

        return DatabaseResource(self)

    @cached_property
    def format(self) -> FormatResource:
        from .resources.format import FormatResource

        return FormatResource(self)

    @cached_property
    def legal(self) -> LegalResource:
        from .resources.legal import LegalResource

        return LegalResource(self)

    @cached_property
    def llm(self) -> LlmResource:
        from .resources.llm import LlmResource

        return LlmResource(self)

    @cached_property
    def memory(self) -> MemoryResource:
        from .resources.memory import MemoryResource

        return MemoryResource(self)

    @cached_property
    def ocr(self) -> OcrResource:
        from .resources.ocr import OcrResource

        return OcrResource(self)

    @cached_property
    def privilege(self) -> PrivilegeResource:
        from .resources.privilege import PrivilegeResource

        return PrivilegeResource(self)

    @cached_property
    def search(self) -> SearchResource:
        from .resources.search import SearchResource

        return SearchResource(self)

    @cached_property
    def superdoc(self) -> SuperdocResource:
        from .resources.superdoc import SuperdocResource

        return SuperdocResource(self)

    @cached_property
    def translate(self) -> TranslateResource:
        from .resources.translate import TranslateResource

        return TranslateResource(self)

    @cached_property
    def vault(self) -> VaultResource:
        from .resources.vault import VaultResource

        return VaultResource(self)

    @cached_property
    def voice(self) -> VoiceResource:
        from .resources.voice import VoiceResource

        return VoiceResource(self)

    @cached_property
    def with_raw_response(self) -> CasedevWithRawResponse:
        return CasedevWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CasedevWithStreamedResponse:
        return CasedevWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        environment: Literal["production", "local"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncCasedev(AsyncAPIClient):
    # client options
    api_key: str

    _environment: Literal["production", "local"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        environment: Literal["production", "local"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncCasedev client instance.

        This automatically infers the `api_key` argument from the `CASEDEV_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("CASEDEV_API_KEY")
        if api_key is None:
            raise CasedevError(
                "The api_key client option must be set either by passing api_key to the client or by setting the CASEDEV_API_KEY environment variable"
            )
        self.api_key = api_key

        self._environment = environment

        base_url_env = os.environ.get("CASEDEV_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `CASEDEV_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def agent(self) -> AsyncAgentResource:
        from .resources.agent import AsyncAgentResource

        return AsyncAgentResource(self)

    @cached_property
    def system(self) -> AsyncSystemResource:
        from .resources.system import AsyncSystemResource

        return AsyncSystemResource(self)

    @cached_property
    def applications(self) -> AsyncApplicationsResource:
        from .resources.applications import AsyncApplicationsResource

        return AsyncApplicationsResource(self)

    @cached_property
    def compute(self) -> AsyncComputeResource:
        from .resources.compute import AsyncComputeResource

        return AsyncComputeResource(self)

    @cached_property
    def database(self) -> AsyncDatabaseResource:
        from .resources.database import AsyncDatabaseResource

        return AsyncDatabaseResource(self)

    @cached_property
    def format(self) -> AsyncFormatResource:
        from .resources.format import AsyncFormatResource

        return AsyncFormatResource(self)

    @cached_property
    def legal(self) -> AsyncLegalResource:
        from .resources.legal import AsyncLegalResource

        return AsyncLegalResource(self)

    @cached_property
    def llm(self) -> AsyncLlmResource:
        from .resources.llm import AsyncLlmResource

        return AsyncLlmResource(self)

    @cached_property
    def memory(self) -> AsyncMemoryResource:
        from .resources.memory import AsyncMemoryResource

        return AsyncMemoryResource(self)

    @cached_property
    def ocr(self) -> AsyncOcrResource:
        from .resources.ocr import AsyncOcrResource

        return AsyncOcrResource(self)

    @cached_property
    def privilege(self) -> AsyncPrivilegeResource:
        from .resources.privilege import AsyncPrivilegeResource

        return AsyncPrivilegeResource(self)

    @cached_property
    def search(self) -> AsyncSearchResource:
        from .resources.search import AsyncSearchResource

        return AsyncSearchResource(self)

    @cached_property
    def superdoc(self) -> AsyncSuperdocResource:
        from .resources.superdoc import AsyncSuperdocResource

        return AsyncSuperdocResource(self)

    @cached_property
    def translate(self) -> AsyncTranslateResource:
        from .resources.translate import AsyncTranslateResource

        return AsyncTranslateResource(self)

    @cached_property
    def vault(self) -> AsyncVaultResource:
        from .resources.vault import AsyncVaultResource

        return AsyncVaultResource(self)

    @cached_property
    def voice(self) -> AsyncVoiceResource:
        from .resources.voice import AsyncVoiceResource

        return AsyncVoiceResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncCasedevWithRawResponse:
        return AsyncCasedevWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCasedevWithStreamedResponse:
        return AsyncCasedevWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        environment: Literal["production", "local"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class CasedevWithRawResponse:
    _client: Casedev

    def __init__(self, client: Casedev) -> None:
        self._client = client

    @cached_property
    def agent(self) -> agent.AgentResourceWithRawResponse:
        from .resources.agent import AgentResourceWithRawResponse

        return AgentResourceWithRawResponse(self._client.agent)

    @cached_property
    def system(self) -> system.SystemResourceWithRawResponse:
        from .resources.system import SystemResourceWithRawResponse

        return SystemResourceWithRawResponse(self._client.system)

    @cached_property
    def applications(self) -> applications.ApplicationsResourceWithRawResponse:
        from .resources.applications import ApplicationsResourceWithRawResponse

        return ApplicationsResourceWithRawResponse(self._client.applications)

    @cached_property
    def compute(self) -> compute.ComputeResourceWithRawResponse:
        from .resources.compute import ComputeResourceWithRawResponse

        return ComputeResourceWithRawResponse(self._client.compute)

    @cached_property
    def database(self) -> database.DatabaseResourceWithRawResponse:
        from .resources.database import DatabaseResourceWithRawResponse

        return DatabaseResourceWithRawResponse(self._client.database)

    @cached_property
    def format(self) -> format.FormatResourceWithRawResponse:
        from .resources.format import FormatResourceWithRawResponse

        return FormatResourceWithRawResponse(self._client.format)

    @cached_property
    def legal(self) -> legal.LegalResourceWithRawResponse:
        from .resources.legal import LegalResourceWithRawResponse

        return LegalResourceWithRawResponse(self._client.legal)

    @cached_property
    def llm(self) -> llm.LlmResourceWithRawResponse:
        from .resources.llm import LlmResourceWithRawResponse

        return LlmResourceWithRawResponse(self._client.llm)

    @cached_property
    def memory(self) -> memory.MemoryResourceWithRawResponse:
        from .resources.memory import MemoryResourceWithRawResponse

        return MemoryResourceWithRawResponse(self._client.memory)

    @cached_property
    def ocr(self) -> ocr.OcrResourceWithRawResponse:
        from .resources.ocr import OcrResourceWithRawResponse

        return OcrResourceWithRawResponse(self._client.ocr)

    @cached_property
    def privilege(self) -> privilege.PrivilegeResourceWithRawResponse:
        from .resources.privilege import PrivilegeResourceWithRawResponse

        return PrivilegeResourceWithRawResponse(self._client.privilege)

    @cached_property
    def search(self) -> search.SearchResourceWithRawResponse:
        from .resources.search import SearchResourceWithRawResponse

        return SearchResourceWithRawResponse(self._client.search)

    @cached_property
    def superdoc(self) -> superdoc.SuperdocResourceWithRawResponse:
        from .resources.superdoc import SuperdocResourceWithRawResponse

        return SuperdocResourceWithRawResponse(self._client.superdoc)

    @cached_property
    def translate(self) -> translate.TranslateResourceWithRawResponse:
        from .resources.translate import TranslateResourceWithRawResponse

        return TranslateResourceWithRawResponse(self._client.translate)

    @cached_property
    def vault(self) -> vault.VaultResourceWithRawResponse:
        from .resources.vault import VaultResourceWithRawResponse

        return VaultResourceWithRawResponse(self._client.vault)

    @cached_property
    def voice(self) -> voice.VoiceResourceWithRawResponse:
        from .resources.voice import VoiceResourceWithRawResponse

        return VoiceResourceWithRawResponse(self._client.voice)


class AsyncCasedevWithRawResponse:
    _client: AsyncCasedev

    def __init__(self, client: AsyncCasedev) -> None:
        self._client = client

    @cached_property
    def agent(self) -> agent.AsyncAgentResourceWithRawResponse:
        from .resources.agent import AsyncAgentResourceWithRawResponse

        return AsyncAgentResourceWithRawResponse(self._client.agent)

    @cached_property
    def system(self) -> system.AsyncSystemResourceWithRawResponse:
        from .resources.system import AsyncSystemResourceWithRawResponse

        return AsyncSystemResourceWithRawResponse(self._client.system)

    @cached_property
    def applications(self) -> applications.AsyncApplicationsResourceWithRawResponse:
        from .resources.applications import AsyncApplicationsResourceWithRawResponse

        return AsyncApplicationsResourceWithRawResponse(self._client.applications)

    @cached_property
    def compute(self) -> compute.AsyncComputeResourceWithRawResponse:
        from .resources.compute import AsyncComputeResourceWithRawResponse

        return AsyncComputeResourceWithRawResponse(self._client.compute)

    @cached_property
    def database(self) -> database.AsyncDatabaseResourceWithRawResponse:
        from .resources.database import AsyncDatabaseResourceWithRawResponse

        return AsyncDatabaseResourceWithRawResponse(self._client.database)

    @cached_property
    def format(self) -> format.AsyncFormatResourceWithRawResponse:
        from .resources.format import AsyncFormatResourceWithRawResponse

        return AsyncFormatResourceWithRawResponse(self._client.format)

    @cached_property
    def legal(self) -> legal.AsyncLegalResourceWithRawResponse:
        from .resources.legal import AsyncLegalResourceWithRawResponse

        return AsyncLegalResourceWithRawResponse(self._client.legal)

    @cached_property
    def llm(self) -> llm.AsyncLlmResourceWithRawResponse:
        from .resources.llm import AsyncLlmResourceWithRawResponse

        return AsyncLlmResourceWithRawResponse(self._client.llm)

    @cached_property
    def memory(self) -> memory.AsyncMemoryResourceWithRawResponse:
        from .resources.memory import AsyncMemoryResourceWithRawResponse

        return AsyncMemoryResourceWithRawResponse(self._client.memory)

    @cached_property
    def ocr(self) -> ocr.AsyncOcrResourceWithRawResponse:
        from .resources.ocr import AsyncOcrResourceWithRawResponse

        return AsyncOcrResourceWithRawResponse(self._client.ocr)

    @cached_property
    def privilege(self) -> privilege.AsyncPrivilegeResourceWithRawResponse:
        from .resources.privilege import AsyncPrivilegeResourceWithRawResponse

        return AsyncPrivilegeResourceWithRawResponse(self._client.privilege)

    @cached_property
    def search(self) -> search.AsyncSearchResourceWithRawResponse:
        from .resources.search import AsyncSearchResourceWithRawResponse

        return AsyncSearchResourceWithRawResponse(self._client.search)

    @cached_property
    def superdoc(self) -> superdoc.AsyncSuperdocResourceWithRawResponse:
        from .resources.superdoc import AsyncSuperdocResourceWithRawResponse

        return AsyncSuperdocResourceWithRawResponse(self._client.superdoc)

    @cached_property
    def translate(self) -> translate.AsyncTranslateResourceWithRawResponse:
        from .resources.translate import AsyncTranslateResourceWithRawResponse

        return AsyncTranslateResourceWithRawResponse(self._client.translate)

    @cached_property
    def vault(self) -> vault.AsyncVaultResourceWithRawResponse:
        from .resources.vault import AsyncVaultResourceWithRawResponse

        return AsyncVaultResourceWithRawResponse(self._client.vault)

    @cached_property
    def voice(self) -> voice.AsyncVoiceResourceWithRawResponse:
        from .resources.voice import AsyncVoiceResourceWithRawResponse

        return AsyncVoiceResourceWithRawResponse(self._client.voice)


class CasedevWithStreamedResponse:
    _client: Casedev

    def __init__(self, client: Casedev) -> None:
        self._client = client

    @cached_property
    def agent(self) -> agent.AgentResourceWithStreamingResponse:
        from .resources.agent import AgentResourceWithStreamingResponse

        return AgentResourceWithStreamingResponse(self._client.agent)

    @cached_property
    def system(self) -> system.SystemResourceWithStreamingResponse:
        from .resources.system import SystemResourceWithStreamingResponse

        return SystemResourceWithStreamingResponse(self._client.system)

    @cached_property
    def applications(self) -> applications.ApplicationsResourceWithStreamingResponse:
        from .resources.applications import ApplicationsResourceWithStreamingResponse

        return ApplicationsResourceWithStreamingResponse(self._client.applications)

    @cached_property
    def compute(self) -> compute.ComputeResourceWithStreamingResponse:
        from .resources.compute import ComputeResourceWithStreamingResponse

        return ComputeResourceWithStreamingResponse(self._client.compute)

    @cached_property
    def database(self) -> database.DatabaseResourceWithStreamingResponse:
        from .resources.database import DatabaseResourceWithStreamingResponse

        return DatabaseResourceWithStreamingResponse(self._client.database)

    @cached_property
    def format(self) -> format.FormatResourceWithStreamingResponse:
        from .resources.format import FormatResourceWithStreamingResponse

        return FormatResourceWithStreamingResponse(self._client.format)

    @cached_property
    def legal(self) -> legal.LegalResourceWithStreamingResponse:
        from .resources.legal import LegalResourceWithStreamingResponse

        return LegalResourceWithStreamingResponse(self._client.legal)

    @cached_property
    def llm(self) -> llm.LlmResourceWithStreamingResponse:
        from .resources.llm import LlmResourceWithStreamingResponse

        return LlmResourceWithStreamingResponse(self._client.llm)

    @cached_property
    def memory(self) -> memory.MemoryResourceWithStreamingResponse:
        from .resources.memory import MemoryResourceWithStreamingResponse

        return MemoryResourceWithStreamingResponse(self._client.memory)

    @cached_property
    def ocr(self) -> ocr.OcrResourceWithStreamingResponse:
        from .resources.ocr import OcrResourceWithStreamingResponse

        return OcrResourceWithStreamingResponse(self._client.ocr)

    @cached_property
    def privilege(self) -> privilege.PrivilegeResourceWithStreamingResponse:
        from .resources.privilege import PrivilegeResourceWithStreamingResponse

        return PrivilegeResourceWithStreamingResponse(self._client.privilege)

    @cached_property
    def search(self) -> search.SearchResourceWithStreamingResponse:
        from .resources.search import SearchResourceWithStreamingResponse

        return SearchResourceWithStreamingResponse(self._client.search)

    @cached_property
    def superdoc(self) -> superdoc.SuperdocResourceWithStreamingResponse:
        from .resources.superdoc import SuperdocResourceWithStreamingResponse

        return SuperdocResourceWithStreamingResponse(self._client.superdoc)

    @cached_property
    def translate(self) -> translate.TranslateResourceWithStreamingResponse:
        from .resources.translate import TranslateResourceWithStreamingResponse

        return TranslateResourceWithStreamingResponse(self._client.translate)

    @cached_property
    def vault(self) -> vault.VaultResourceWithStreamingResponse:
        from .resources.vault import VaultResourceWithStreamingResponse

        return VaultResourceWithStreamingResponse(self._client.vault)

    @cached_property
    def voice(self) -> voice.VoiceResourceWithStreamingResponse:
        from .resources.voice import VoiceResourceWithStreamingResponse

        return VoiceResourceWithStreamingResponse(self._client.voice)


class AsyncCasedevWithStreamedResponse:
    _client: AsyncCasedev

    def __init__(self, client: AsyncCasedev) -> None:
        self._client = client

    @cached_property
    def agent(self) -> agent.AsyncAgentResourceWithStreamingResponse:
        from .resources.agent import AsyncAgentResourceWithStreamingResponse

        return AsyncAgentResourceWithStreamingResponse(self._client.agent)

    @cached_property
    def system(self) -> system.AsyncSystemResourceWithStreamingResponse:
        from .resources.system import AsyncSystemResourceWithStreamingResponse

        return AsyncSystemResourceWithStreamingResponse(self._client.system)

    @cached_property
    def applications(self) -> applications.AsyncApplicationsResourceWithStreamingResponse:
        from .resources.applications import AsyncApplicationsResourceWithStreamingResponse

        return AsyncApplicationsResourceWithStreamingResponse(self._client.applications)

    @cached_property
    def compute(self) -> compute.AsyncComputeResourceWithStreamingResponse:
        from .resources.compute import AsyncComputeResourceWithStreamingResponse

        return AsyncComputeResourceWithStreamingResponse(self._client.compute)

    @cached_property
    def database(self) -> database.AsyncDatabaseResourceWithStreamingResponse:
        from .resources.database import AsyncDatabaseResourceWithStreamingResponse

        return AsyncDatabaseResourceWithStreamingResponse(self._client.database)

    @cached_property
    def format(self) -> format.AsyncFormatResourceWithStreamingResponse:
        from .resources.format import AsyncFormatResourceWithStreamingResponse

        return AsyncFormatResourceWithStreamingResponse(self._client.format)

    @cached_property
    def legal(self) -> legal.AsyncLegalResourceWithStreamingResponse:
        from .resources.legal import AsyncLegalResourceWithStreamingResponse

        return AsyncLegalResourceWithStreamingResponse(self._client.legal)

    @cached_property
    def llm(self) -> llm.AsyncLlmResourceWithStreamingResponse:
        from .resources.llm import AsyncLlmResourceWithStreamingResponse

        return AsyncLlmResourceWithStreamingResponse(self._client.llm)

    @cached_property
    def memory(self) -> memory.AsyncMemoryResourceWithStreamingResponse:
        from .resources.memory import AsyncMemoryResourceWithStreamingResponse

        return AsyncMemoryResourceWithStreamingResponse(self._client.memory)

    @cached_property
    def ocr(self) -> ocr.AsyncOcrResourceWithStreamingResponse:
        from .resources.ocr import AsyncOcrResourceWithStreamingResponse

        return AsyncOcrResourceWithStreamingResponse(self._client.ocr)

    @cached_property
    def privilege(self) -> privilege.AsyncPrivilegeResourceWithStreamingResponse:
        from .resources.privilege import AsyncPrivilegeResourceWithStreamingResponse

        return AsyncPrivilegeResourceWithStreamingResponse(self._client.privilege)

    @cached_property
    def search(self) -> search.AsyncSearchResourceWithStreamingResponse:
        from .resources.search import AsyncSearchResourceWithStreamingResponse

        return AsyncSearchResourceWithStreamingResponse(self._client.search)

    @cached_property
    def superdoc(self) -> superdoc.AsyncSuperdocResourceWithStreamingResponse:
        from .resources.superdoc import AsyncSuperdocResourceWithStreamingResponse

        return AsyncSuperdocResourceWithStreamingResponse(self._client.superdoc)

    @cached_property
    def translate(self) -> translate.AsyncTranslateResourceWithStreamingResponse:
        from .resources.translate import AsyncTranslateResourceWithStreamingResponse

        return AsyncTranslateResourceWithStreamingResponse(self._client.translate)

    @cached_property
    def vault(self) -> vault.AsyncVaultResourceWithStreamingResponse:
        from .resources.vault import AsyncVaultResourceWithStreamingResponse

        return AsyncVaultResourceWithStreamingResponse(self._client.vault)

    @cached_property
    def voice(self) -> voice.AsyncVoiceResourceWithStreamingResponse:
        from .resources.voice import AsyncVoiceResourceWithStreamingResponse

        return AsyncVoiceResourceWithStreamingResponse(self._client.voice)


Client = Casedev

AsyncClient = AsyncCasedev



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from casedev import Casedev, AsyncCasedev
from tests.utils import assert_matches_type
from casedev.types.voice import V1ListVoicesResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestV1:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list_voices(self, client: Casedev) -> None:
        v1 = client.voice.v1.list_voices()
        assert_matches_type(V1ListVoicesResponse, v1, path=["response"])

    @parametrize
    def test_method_list_voices_with_all_params(self, client: Casedev) -> None:
        v1 = client.voice.v1.list_voices(
            category="category",
            collection_id="collection_id",
            include_total_count=True,
            next_page_token="next_page_token",
            page_size=1,
            search="search",
            sort="name",
            sort_direction="asc",
            voice_type="premade",
        )
        assert_matches_type(V1ListVoicesResponse, v1, path=["response"])

    @parametrize
    def test_raw_response_list_voices(self, client: Casedev) -> None:
        response = client.voice.v1.with_raw_response.list_voices()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        v1 = response.parse()
        assert_matches_type(V1ListVoicesResponse, v1, path=["response"])

    @parametrize
    def test_streaming_response_list_voices(self, client: Casedev) -> None:
        with client.voice.v1.with_streaming_response.list_voices() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            v1 = response.parse()
            assert_matches_type(V1ListVoicesResponse, v1, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncV1:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_list_voices(self, async_client: AsyncCasedev) -> None:
        v1 = await async_client.voice.v1.list_voices()
        assert_matches_type(V1ListVoicesResponse, v1, path=["response"])

    @parametrize
    async def test_method_list_voices_with_all_params(self, async_client: AsyncCasedev) -> None:
        v1 = await async_client.voice.v1.list_voices(
            category="category",
            collection_id="collection_id",
            include_total_count=True,
            next_page_token="next_page_token",
            page_size=1,
            search="search",
            sort="name",
            sort_direction="asc",
            voice_type="premade",
        )
        assert_matches_type(V1ListVoicesResponse, v1, path=["response"])

    @parametrize
    async def test_raw_response_list_voices(self, async_client: AsyncCasedev) -> None:
        response = await async_client.voice.v1.with_raw_response.list_voices()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        v1 = await response.parse()
        assert_matches_type(V1ListVoicesResponse, v1, path=["response"])

    @parametrize
    async def test_streaming_response_list_voices(self, async_client: AsyncCasedev) -> None:
        async with async_client.voice.v1.with_streaming_response.list_voices() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            v1 = await response.parse()
            assert_matches_type(V1ListVoicesResponse, v1, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .v1_list_models_response import V1ListModelsResponse as V1ListModelsResponse
from .v1_create_embedding_params import V1CreateEmbeddingParams as V1CreateEmbeddingParams
from .v1_create_embedding_response import V1CreateEmbeddingResponse as V1CreateEmbeddingResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["V1ContentsResponse", "Result"]


class Result(BaseModel):
    highlights: Optional[List[str]] = None
    """Content highlights if requested"""

    metadata: Optional[object] = None
    """Additional metadata about the content"""

    summary: Optional[str] = None
    """Content summary if requested"""

    text: Optional[str] = None
    """Extracted text content"""

    title: Optional[str] = None
    """Page title"""

    url: Optional[str] = None
    """Source URL"""


class V1ContentsResponse(BaseModel):
    results: Optional[List[Result]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.memory import v1_list_params, v1_create_params, v1_search_params, v1_delete_all_params
from ...types.memory.v1_list_response import V1ListResponse
from ...types.memory.v1_create_response import V1CreateResponse
from ...types.memory.v1_delete_response import V1DeleteResponse
from ...types.memory.v1_search_response import V1SearchResponse
from ...types.memory.v1_retrieve_response import V1RetrieveResponse
from ...types.memory.v1_delete_all_response import V1DeleteAllResponse

__all__ = ["V1Resource", "AsyncV1Resource"]


class V1Resource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> V1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return V1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> V1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return V1ResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        messages: Iterable[v1_create_params.Message],
        category: str | Omit = omit,
        extraction_prompt: str | Omit = omit,
        infer: bool | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        tag_1: str | Omit = omit,
        tag_10: str | Omit = omit,
        tag_11: str | Omit = omit,
        tag_12: str | Omit = omit,
        tag_2: str | Omit = omit,
        tag_3: str | Omit = omit,
        tag_4: str | Omit = omit,
        tag_5: str | Omit = omit,
        tag_6: str | Omit = omit,
        tag_7: str | Omit = omit,
        tag_8: str | Omit = omit,
        tag_9: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1CreateResponse:
        """Store memories from conversation messages.

        Automatically extracts facts and
        handles deduplication.

        Use tag_1 through tag_12 for filtering - these are generic indexed fields you
        can use for any purpose:

        - Legal app: tag_1=client_id, tag_2=matter_id
        - Healthcare: tag_1=patient_id, tag_2=encounter_id
        - E-commerce: tag_1=customer_id, tag_2=order_id

        Args:
          messages: Conversation messages to extract memories from

          category: Custom category (e.g., "fact", "preference", "deadline")

          extraction_prompt: Optional custom prompt for fact extraction

          infer: Whether to extract facts from messages (default: true)

          metadata: Additional metadata (not indexed)

          tag_1: Generic indexed filter field 1 (you decide what it means)

          tag_10: Generic indexed filter field 10

          tag_11: Generic indexed filter field 11

          tag_12: Generic indexed filter field 12

          tag_2: Generic indexed filter field 2

          tag_3: Generic indexed filter field 3

          tag_4: Generic indexed filter field 4

          tag_5: Generic indexed filter field 5

          tag_6: Generic indexed filter field 6

          tag_7: Generic indexed filter field 7

          tag_8: Generic indexed filter field 8

          tag_9: Generic indexed filter field 9

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/memory/v1",
            body=maybe_transform(
                {
                    "messages": messages,
                    "category": category,
                    "extraction_prompt": extraction_prompt,
                    "infer": infer,
                    "metadata": metadata,
                    "tag_1": tag_1,
                    "tag_10": tag_10,
                    "tag_11": tag_11,
                    "tag_12": tag_12,
                    "tag_2": tag_2,
                    "tag_3": tag_3,
                    "tag_4": tag_4,
                    "tag_5": tag_5,
                    "tag_6": tag_6,
                    "tag_7": tag_7,
                    "tag_8": tag_8,
                    "tag_9": tag_9,
                },
                v1_create_params.V1CreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1CreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1RetrieveResponse:
        """
        Retrieve a single memory by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/memory/v1/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1RetrieveResponse,
        )

    def list(
        self,
        *,
        category: str | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        tag_1: str | Omit = omit,
        tag_10: str | Omit = omit,
        tag_11: str | Omit = omit,
        tag_12: str | Omit = omit,
        tag_2: str | Omit = omit,
        tag_3: str | Omit = omit,
        tag_4: str | Omit = omit,
        tag_5: str | Omit = omit,
        tag_6: str | Omit = omit,
        tag_7: str | Omit = omit,
        tag_8: str | Omit = omit,
        tag_9: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1ListResponse:
        """
        List all memories with optional filtering by tags and category.

        Args:
          category: Filter by category

          limit: Number of results

          offset: Pagination offset

          tag_1: Filter by tag_1

          tag_10: Filter by tag_10

          tag_11: Filter by tag_11

          tag_12: Filter by tag_12

          tag_2: Filter by tag_2

          tag_3: Filter by tag_3

          tag_4: Filter by tag_4

          tag_5: Filter by tag_5

          tag_6: Filter by tag_6

          tag_7: Filter by tag_7

          tag_8: Filter by tag_8

          tag_9: Filter by tag_9

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/memory/v1",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "category": category,
                        "limit": limit,
                        "offset": offset,
                        "tag_1": tag_1,
                        "tag_10": tag_10,
                        "tag_11": tag_11,
                        "tag_12": tag_12,
                        "tag_2": tag_2,
                        "tag_3": tag_3,
                        "tag_4": tag_4,
                        "tag_5": tag_5,
                        "tag_6": tag_6,
                        "tag_7": tag_7,
                        "tag_8": tag_8,
                        "tag_9": tag_9,
                    },
                    v1_list_params.V1ListParams,
                ),
            ),
            cast_to=V1ListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1DeleteResponse:
        """
        Delete a single memory by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/memory/v1/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1DeleteResponse,
        )

    def delete_all(
        self,
        *,
        tag_1: str | Omit = omit,
        tag_10: str | Omit = omit,
        tag_11: str | Omit = omit,
        tag_12: str | Omit = omit,
        tag_2: str | Omit = omit,
        tag_3: str | Omit = omit,
        tag_4: str | Omit = omit,
        tag_5: str | Omit = omit,
        tag_6: str | Omit = omit,
        tag_7: str | Omit = omit,
        tag_8: str | Omit = omit,
        tag_9: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1DeleteAllResponse:
        """Delete multiple memories matching tag filter criteria.

        CAUTION: This will delete
        all matching memories for your organization.

        Args:
          tag_1: Filter by tag_1

          tag_10: Filter by tag_10

          tag_11: Filter by tag_11

          tag_12: Filter by tag_12

          tag_2: Filter by tag_2

          tag_3: Filter by tag_3

          tag_4: Filter by tag_4

          tag_5: Filter by tag_5

          tag_6: Filter by tag_6

          tag_7: Filter by tag_7

          tag_8: Filter by tag_8

          tag_9: Filter by tag_9

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            "/memory/v1",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "tag_1": tag_1,
                        "tag_10": tag_10,
                        "tag_11": tag_11,
                        "tag_12": tag_12,
                        "tag_2": tag_2,
                        "tag_3": tag_3,
                        "tag_4": tag_4,
                        "tag_5": tag_5,
                        "tag_6": tag_6,
                        "tag_7": tag_7,
                        "tag_8": tag_8,
                        "tag_9": tag_9,
                    },
                    v1_delete_all_params.V1DeleteAllParams,
                ),
            ),
            cast_to=V1DeleteAllResponse,
        )

    def search(
        self,
        *,
        query: str,
        category: str | Omit = omit,
        tag_1: str | Omit = omit,
        tag_10: str | Omit = omit,
        tag_11: str | Omit = omit,
        tag_12: str | Omit = omit,
        tag_2: str | Omit = omit,
        tag_3: str | Omit = omit,
        tag_4: str | Omit = omit,
        tag_5: str | Omit = omit,
        tag_6: str | Omit = omit,
        tag_7: str | Omit = omit,
        tag_8: str | Omit = omit,
        tag_9: str | Omit = omit,
        top_k: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1SearchResponse:
        """Search memories using semantic similarity.

        Filter by tag fields to narrow
        results.

        Use tag_1 through tag_12 for filtering - these are generic indexed fields you
        define:

        - Legal app: tag_1=client_id, tag_2=matter_id
        - Healthcare: tag_1=patient_id, tag_2=encounter_id

        Args:
          query: Search query for semantic matching

          category: Filter by category

          tag_1: Filter by tag_1

          tag_10: Filter by tag_10

          tag_11: Filter by tag_11

          tag_12: Filter by tag_12

          tag_2: Filter by tag_2

          tag_3: Filter by tag_3

          tag_4: Filter by tag_4

          tag_5: Filter by tag_5

          tag_6: Filter by tag_6

          tag_7: Filter by tag_7

          tag_8: Filter by tag_8

          tag_9: Filter by tag_9

          top_k: Maximum number of results to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/memory/v1/search",
            body=maybe_transform(
                {
                    "query": query,
                    "category": category,
                    "tag_1": tag_1,
                    "tag_10": tag_10,
                    "tag_11": tag_11,
                    "tag_12": tag_12,
                    "tag_2": tag_2,
                    "tag_3": tag_3,
                    "tag_4": tag_4,
                    "tag_5": tag_5,
                    "tag_6": tag_6,
                    "tag_7": tag_7,
                    "tag_8": tag_8,
                    "tag_9": tag_9,
                    "top_k": top_k,
                },
                v1_search_params.V1SearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1SearchResponse,
        )


class AsyncV1Resource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncV1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return AsyncV1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncV1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return AsyncV1ResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        messages: Iterable[v1_create_params.Message],
        category: str | Omit = omit,
        extraction_prompt: str | Omit = omit,
        infer: bool | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        tag_1: str | Omit = omit,
        tag_10: str | Omit = omit,
        tag_11: str | Omit = omit,
        tag_12: str | Omit = omit,
        tag_2: str | Omit = omit,
        tag_3: str | Omit = omit,
        tag_4: str | Omit = omit,
        tag_5: str | Omit = omit,
        tag_6: str | Omit = omit,
        tag_7: str | Omit = omit,
        tag_8: str | Omit = omit,
        tag_9: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1CreateResponse:
        """Store memories from conversation messages.

        Automatically extracts facts and
        handles deduplication.

        Use tag_1 through tag_12 for filtering - these are generic indexed fields you
        can use for any purpose:

        - Legal app: tag_1=client_id, tag_2=matter_id
        - Healthcare: tag_1=patient_id, tag_2=encounter_id
        - E-commerce: tag_1=customer_id, tag_2=order_id

        Args:
          messages: Conversation messages to extract memories from

          category: Custom category (e.g., "fact", "preference", "deadline")

          extraction_prompt: Optional custom prompt for fact extraction

          infer: Whether to extract facts from messages (default: true)

          metadata: Additional metadata (not indexed)

          tag_1: Generic indexed filter field 1 (you decide what it means)

          tag_10: Generic indexed filter field 10

          tag_11: Generic indexed filter field 11

          tag_12: Generic indexed filter field 12

          tag_2: Generic indexed filter field 2

          tag_3: Generic indexed filter field 3

          tag_4: Generic indexed filter field 4

          tag_5: Generic indexed filter field 5

          tag_6: Generic indexed filter field 6

          tag_7: Generic indexed filter field 7

          tag_8: Generic indexed filter field 8

          tag_9: Generic indexed filter field 9

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/memory/v1",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "category": category,
                    "extraction_prompt": extraction_prompt,
                    "infer": infer,
                    "metadata": metadata,
                    "tag_1": tag_1,
                    "tag_10": tag_10,
                    "tag_11": tag_11,
                    "tag_12": tag_12,
                    "tag_2": tag_2,
                    "tag_3": tag_3,
                    "tag_4": tag_4,
                    "tag_5": tag_5,
                    "tag_6": tag_6,
                    "tag_7": tag_7,
                    "tag_8": tag_8,
                    "tag_9": tag_9,
                },
                v1_create_params.V1CreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1CreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1RetrieveResponse:
        """
        Retrieve a single memory by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/memory/v1/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1RetrieveResponse,
        )

    async def list(
        self,
        *,
        category: str | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        tag_1: str | Omit = omit,
        tag_10: str | Omit = omit,
        tag_11: str | Omit = omit,
        tag_12: str | Omit = omit,
        tag_2: str | Omit = omit,
        tag_3: str | Omit = omit,
        tag_4: str | Omit = omit,
        tag_5: str | Omit = omit,
        tag_6: str | Omit = omit,
        tag_7: str | Omit = omit,
        tag_8: str | Omit = omit,
        tag_9: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1ListResponse:
        """
        List all memories with optional filtering by tags and category.

        Args:
          category: Filter by category

          limit: Number of results

          offset: Pagination offset

          tag_1: Filter by tag_1

          tag_10: Filter by tag_10

          tag_11: Filter by tag_11

          tag_12: Filter by tag_12

          tag_2: Filter by tag_2

          tag_3: Filter by tag_3

          tag_4: Filter by tag_4

          tag_5: Filter by tag_5

          tag_6: Filter by tag_6

          tag_7: Filter by tag_7

          tag_8: Filter by tag_8

          tag_9: Filter by tag_9

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/memory/v1",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "category": category,
                        "limit": limit,
                        "offset": offset,
                        "tag_1": tag_1,
                        "tag_10": tag_10,
                        "tag_11": tag_11,
                        "tag_12": tag_12,
                        "tag_2": tag_2,
                        "tag_3": tag_3,
                        "tag_4": tag_4,
                        "tag_5": tag_5,
                        "tag_6": tag_6,
                        "tag_7": tag_7,
                        "tag_8": tag_8,
                        "tag_9": tag_9,
                    },
                    v1_list_params.V1ListParams,
                ),
            ),
            cast_to=V1ListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1DeleteResponse:
        """
        Delete a single memory by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/memory/v1/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1DeleteResponse,
        )

    async def delete_all(
        self,
        *,
        tag_1: str | Omit = omit,
        tag_10: str | Omit = omit,
        tag_11: str | Omit = omit,
        tag_12: str | Omit = omit,
        tag_2: str | Omit = omit,
        tag_3: str | Omit = omit,
        tag_4: str | Omit = omit,
        tag_5: str | Omit = omit,
        tag_6: str | Omit = omit,
        tag_7: str | Omit = omit,
        tag_8: str | Omit = omit,
        tag_9: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1DeleteAllResponse:
        """Delete multiple memories matching tag filter criteria.

        CAUTION: This will delete
        all matching memories for your organization.

        Args:
          tag_1: Filter by tag_1

          tag_10: Filter by tag_10

          tag_11: Filter by tag_11

          tag_12: Filter by tag_12

          tag_2: Filter by tag_2

          tag_3: Filter by tag_3

          tag_4: Filter by tag_4

          tag_5: Filter by tag_5

          tag_6: Filter by tag_6

          tag_7: Filter by tag_7

          tag_8: Filter by tag_8

          tag_9: Filter by tag_9

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            "/memory/v1",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "tag_1": tag_1,
                        "tag_10": tag_10,
                        "tag_11": tag_11,
                        "tag_12": tag_12,
                        "tag_2": tag_2,
                        "tag_3": tag_3,
                        "tag_4": tag_4,
                        "tag_5": tag_5,
                        "tag_6": tag_6,
                        "tag_7": tag_7,
                        "tag_8": tag_8,
                        "tag_9": tag_9,
                    },
                    v1_delete_all_params.V1DeleteAllParams,
                ),
            ),
            cast_to=V1DeleteAllResponse,
        )

    async def search(
        self,
        *,
        query: str,
        category: str | Omit = omit,
        tag_1: str | Omit = omit,
        tag_10: str | Omit = omit,
        tag_11: str | Omit = omit,
        tag_12: str | Omit = omit,
        tag_2: str | Omit = omit,
        tag_3: str | Omit = omit,
        tag_4: str | Omit = omit,
        tag_5: str | Omit = omit,
        tag_6: str | Omit = omit,
        tag_7: str | Omit = omit,
        tag_8: str | Omit = omit,
        tag_9: str | Omit = omit,
        top_k: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1SearchResponse:
        """Search memories using semantic similarity.

        Filter by tag fields to narrow
        results.

        Use tag_1 through tag_12 for filtering - these are generic indexed fields you
        define:

        - Legal app: tag_1=client_id, tag_2=matter_id
        - Healthcare: tag_1=patient_id, tag_2=encounter_id

        Args:
          query: Search query for semantic matching

          category: Filter by category

          tag_1: Filter by tag_1

          tag_10: Filter by tag_10

          tag_11: Filter by tag_11

          tag_12: Filter by tag_12

          tag_2: Filter by tag_2

          tag_3: Filter by tag_3

          tag_4: Filter by tag_4

          tag_5: Filter by tag_5

          tag_6: Filter by tag_6

          tag_7: Filter by tag_7

          tag_8: Filter by tag_8

          tag_9: Filter by tag_9

          top_k: Maximum number of results to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/memory/v1/search",
            body=await async_maybe_transform(
                {
                    "query": query,
                    "category": category,
                    "tag_1": tag_1,
                    "tag_10": tag_10,
                    "tag_11": tag_11,
                    "tag_12": tag_12,
                    "tag_2": tag_2,
                    "tag_3": tag_3,
                    "tag_4": tag_4,
                    "tag_5": tag_5,
                    "tag_6": tag_6,
                    "tag_7": tag_7,
                    "tag_8": tag_8,
                    "tag_9": tag_9,
                    "top_k": top_k,
                },
                v1_search_params.V1SearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1SearchResponse,
        )


class V1ResourceWithRawResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

        self.create = to_raw_response_wrapper(
            v1.create,
        )
        self.retrieve = to_raw_response_wrapper(
            v1.retrieve,
        )
        self.list = to_raw_response_wrapper(
            v1.list,
        )
        self.delete = to_raw_response_wrapper(
            v1.delete,
        )
        self.delete_all = to_raw_response_wrapper(
            v1.delete_all,
        )
        self.search = to_raw_response_wrapper(
            v1.search,
        )


class AsyncV1ResourceWithRawResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

        self.create = async_to_raw_response_wrapper(
            v1.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            v1.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            v1.list,
        )
        self.delete = async_to_raw_response_wrapper(
            v1.delete,
        )
        self.delete_all = async_to_raw_response_wrapper(
            v1.delete_all,
        )
        self.search = async_to_raw_response_wrapper(
            v1.search,
        )


class V1ResourceWithStreamingResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

        self.create = to_streamed_response_wrapper(
            v1.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            v1.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            v1.list,
        )
        self.delete = to_streamed_response_wrapper(
            v1.delete,
        )
        self.delete_all = to_streamed_response_wrapper(
            v1.delete_all,
        )
        self.search = to_streamed_response_wrapper(
            v1.search,
        )


class AsyncV1ResourceWithStreamingResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

        self.create = async_to_streamed_response_wrapper(
            v1.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            v1.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            v1.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            v1.delete,
        )
        self.delete_all = async_to_streamed_response_wrapper(
            v1.delete_all,
        )
        self.search = async_to_streamed_response_wrapper(
            v1.search,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ObjectUpdateResponse"]


class ObjectUpdateResponse(BaseModel):
    id: Optional[str] = None
    """Object ID"""

    content_type: Optional[str] = FieldInfo(alias="contentType", default=None)
    """MIME type"""

    filename: Optional[str] = None
    """Updated filename"""

    ingestion_status: Optional[str] = FieldInfo(alias="ingestionStatus", default=None)
    """Processing status"""

    metadata: Optional[object] = None
    """Full metadata object"""

    path: Optional[str] = None
    """Folder path for hierarchy preservation"""

    size_bytes: Optional[int] = FieldInfo(alias="sizeBytes", default=None)
    """File size in bytes"""

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)
    """Last update timestamp"""

    vault_id: Optional[str] = FieldInfo(alias="vaultId", default=None)
    """Vault ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["AgentUpdateParams"]


class AgentUpdateParams(TypedDict, total=False):
    description: str

    disabled_tools: Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="disabledTools")]

    enabled_tools: Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="enabledTools")]

    instructions: str

    model: str

    name: str

    sandbox: Optional[object]

    vault_groups: Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="vaultGroups")]

    vault_ids: Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="vaultIds")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["V1CreateEmbeddingParams"]


class V1CreateEmbeddingParams(TypedDict, total=False):
    input: Required[Union[str, SequenceNotStr[str]]]
    """Text or array of texts to create embeddings for"""

    model: Required[str]
    """Embedding model to use (e.g., text-embedding-ada-002, text-embedding-3-small)"""

    dimensions: int
    """Number of dimensions for the embeddings (model-specific)"""

    encoding_format: Literal["float", "base64"]
    """Format for returned embeddings"""

    user: str
    """Unique identifier for the end-user"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .run import (
    RunResource,
    AsyncRunResource,
    RunResourceWithRawResponse,
    AsyncRunResourceWithRawResponse,
    RunResourceWithStreamingResponse,
    AsyncRunResourceWithStreamingResponse,
)
from .agents import (
    AgentsResource,
    AsyncAgentsResource,
    AgentsResourceWithRawResponse,
    AsyncAgentsResourceWithRawResponse,
    AgentsResourceWithStreamingResponse,
    AsyncAgentsResourceWithStreamingResponse,
)
from .execute import (
    ExecuteResource,
    AsyncExecuteResource,
    ExecuteResourceWithRawResponse,
    AsyncExecuteResourceWithRawResponse,
    ExecuteResourceWithStreamingResponse,
    AsyncExecuteResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["V1Resource", "AsyncV1Resource"]


class V1Resource(SyncAPIResource):
    @cached_property
    def agents(self) -> AgentsResource:
        return AgentsResource(self._client)

    @cached_property
    def run(self) -> RunResource:
        return RunResource(self._client)

    @cached_property
    def execute(self) -> ExecuteResource:
        return ExecuteResource(self._client)

    @cached_property
    def with_raw_response(self) -> V1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return V1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> V1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return V1ResourceWithStreamingResponse(self)


class AsyncV1Resource(AsyncAPIResource):
    @cached_property
    def agents(self) -> AsyncAgentsResource:
        return AsyncAgentsResource(self._client)

    @cached_property
    def run(self) -> AsyncRunResource:
        return AsyncRunResource(self._client)

    @cached_property
    def execute(self) -> AsyncExecuteResource:
        return AsyncExecuteResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncV1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return AsyncV1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncV1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return AsyncV1ResourceWithStreamingResponse(self)


class V1ResourceWithRawResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

    @cached_property
    def agents(self) -> AgentsResourceWithRawResponse:
        return AgentsResourceWithRawResponse(self._v1.agents)

    @cached_property
    def run(self) -> RunResourceWithRawResponse:
        return RunResourceWithRawResponse(self._v1.run)

    @cached_property
    def execute(self) -> ExecuteResourceWithRawResponse:
        return ExecuteResourceWithRawResponse(self._v1.execute)


class AsyncV1ResourceWithRawResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

    @cached_property
    def agents(self) -> AsyncAgentsResourceWithRawResponse:
        return AsyncAgentsResourceWithRawResponse(self._v1.agents)

    @cached_property
    def run(self) -> AsyncRunResourceWithRawResponse:
        return AsyncRunResourceWithRawResponse(self._v1.run)

    @cached_property
    def execute(self) -> AsyncExecuteResourceWithRawResponse:
        return AsyncExecuteResourceWithRawResponse(self._v1.execute)


class V1ResourceWithStreamingResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

    @cached_property
    def agents(self) -> AgentsResourceWithStreamingResponse:
        return AgentsResourceWithStreamingResponse(self._v1.agents)

    @cached_property
    def run(self) -> RunResourceWithStreamingResponse:
        return RunResourceWithStreamingResponse(self._v1.run)

    @cached_property
    def execute(self) -> ExecuteResourceWithStreamingResponse:
        return ExecuteResourceWithStreamingResponse(self._v1.execute)


class AsyncV1ResourceWithStreamingResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

    @cached_property
    def agents(self) -> AsyncAgentsResourceWithStreamingResponse:
        return AsyncAgentsResourceWithStreamingResponse(self._v1.agents)

    @cached_property
    def run(self) -> AsyncRunResourceWithStreamingResponse:
        return AsyncRunResourceWithStreamingResponse(self._v1.run)

    @cached_property
    def execute(self) -> AsyncExecuteResourceWithStreamingResponse:
        return AsyncExecuteResourceWithStreamingResponse(self._v1.execute)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .v1 import (
    V1Resource,
    AsyncV1Resource,
    V1ResourceWithRawResponse,
    AsyncV1ResourceWithRawResponse,
    V1ResourceWithStreamingResponse,
    AsyncV1ResourceWithStreamingResponse,
)
from .voice import (
    VoiceResource,
    AsyncVoiceResource,
    VoiceResourceWithRawResponse,
    AsyncVoiceResourceWithRawResponse,
    VoiceResourceWithStreamingResponse,
    AsyncVoiceResourceWithStreamingResponse,
)
from .streaming import (
    StreamingResource,
    AsyncStreamingResource,
    StreamingResourceWithRawResponse,
    AsyncStreamingResourceWithRawResponse,
    StreamingResourceWithStreamingResponse,
    AsyncStreamingResourceWithStreamingResponse,
)
from .transcription import (
    TranscriptionResource,
    AsyncTranscriptionResource,
    TranscriptionResourceWithRawResponse,
    AsyncTranscriptionResourceWithRawResponse,
    TranscriptionResourceWithStreamingResponse,
    AsyncTranscriptionResourceWithStreamingResponse,
)

__all__ = [
    "StreamingResource",
    "AsyncStreamingResource",
    "StreamingResourceWithRawResponse",
    "AsyncStreamingResourceWithRawResponse",
    "StreamingResourceWithStreamingResponse",
    "AsyncStreamingResourceWithStreamingResponse",
    "TranscriptionResource",
    "AsyncTranscriptionResource",
    "TranscriptionResourceWithRawResponse",
    "AsyncTranscriptionResourceWithRawResponse",
    "TranscriptionResourceWithStreamingResponse",
    "AsyncTranscriptionResourceWithStreamingResponse",
    "V1Resource",
    "AsyncV1Resource",
    "V1ResourceWithRawResponse",
    "AsyncV1ResourceWithRawResponse",
    "V1ResourceWithStreamingResponse",
    "AsyncV1ResourceWithStreamingResponse",
    "VoiceResource",
    "AsyncVoiceResource",
    "VoiceResourceWithRawResponse",
    "AsyncVoiceResourceWithRawResponse",
    "VoiceResourceWithStreamingResponse",
    "AsyncVoiceResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .run_watch_params import RunWatchParams as RunWatchParams
from .run_create_params import RunCreateParams as RunCreateParams
from .run_exec_response import RunExecResponse as RunExecResponse
from .run_watch_response import RunWatchResponse as RunWatchResponse
from .agent_create_params import AgentCreateParams as AgentCreateParams
from .agent_list_response import AgentListResponse as AgentListResponse
from .agent_update_params import AgentUpdateParams as AgentUpdateParams
from .run_cancel_response import RunCancelResponse as RunCancelResponse
from .run_create_response import RunCreateResponse as RunCreateResponse
from .agent_create_response import AgentCreateResponse as AgentCreateResponse
from .agent_delete_response import AgentDeleteResponse as AgentDeleteResponse
from .agent_update_response import AgentUpdateResponse as AgentUpdateResponse
from .execute_create_params import ExecuteCreateParams as ExecuteCreateParams
from .agent_retrieve_response import AgentRetrieveResponse as AgentRetrieveResponse
from .execute_create_response import ExecuteCreateResponse as ExecuteCreateResponse
from .run_get_status_response import RunGetStatusResponse as RunGetStatusResponse
from .run_get_details_response import RunGetDetailsResponse as RunGetDetailsResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["V1ListLanguagesParams"]


class V1ListLanguagesParams(TypedDict, total=False):
    model: Literal["nmt", "base"]
    """Translation model to check language support for"""

    target: str
    """
    Target language code for translating language names (e.g., 'es' for Spanish
    names)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["VaultIngestResponse"]


class VaultIngestResponse(BaseModel):
    enable_graph_rag: bool = FieldInfo(alias="enableGraphRAG")
    """
    Always false - GraphRAG must be triggered separately via POST
    /vault/:id/graphrag/:objectId
    """

    message: str
    """Human-readable status message"""

    object_id: str = FieldInfo(alias="objectId")
    """ID of the vault object being processed"""

    status: Literal["processing", "stored"]
    """Current ingestion status.

    'stored' for file types without text extraction (no chunks/vectors created).
    """

    workflow_id: Optional[str] = FieldInfo(alias="workflowId", default=None)
    """Workflow run ID for tracking progress.

    Null for file types that skip processing.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["V1DetectResponse", "Data", "DataDetection"]


class DataDetection(BaseModel):
    confidence: Optional[float] = None
    """Confidence score (0-1)"""

    is_reliable: Optional[bool] = FieldInfo(alias="isReliable", default=None)
    """Whether the detection is reliable"""

    language: Optional[str] = None
    """Detected language code (ISO 639-1)"""


class Data(BaseModel):
    detections: Optional[List[List[DataDetection]]] = None


class V1DetectResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["V1ListJurisdictionsParams"]


class V1ListJurisdictionsParams(TypedDict, total=False):
    name: Required[str]
    """Jurisdiction name (e.g., "California", "US Federal", "NY")"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["RunWatchParams"]


class RunWatchParams(TypedDict, total=False):
    callback_url: Required[Annotated[str, PropertyInfo(alias="callbackUrl")]]
    """HTTPS URL to receive completion callback"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ProjectCreateDeploymentParams", "EnvironmentVariable"]


class ProjectCreateDeploymentParams(TypedDict, total=False):
    environment_variables: Annotated[Iterable[EnvironmentVariable], PropertyInfo(alias="environmentVariables")]
    """Additional environment variables to set or update before deployment"""


class EnvironmentVariable(TypedDict, total=False):
    key: Required[str]
    """Environment variable name"""

    target: Required[List[Literal["production", "preview", "development"]]]
    """Deployment targets for this variable"""

    value: Required[str]
    """Environment variable value"""

    type: Literal["plain", "encrypted", "secret"]
    """Variable type"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["RunCancelResponse"]


class RunCancelResponse(BaseModel):
    id: Optional[str] = None

    message: Optional[str] = None
    """Present if run was already finished"""

    status: Optional[Literal["cancelled", "completed", "failed"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .project_create_params import ProjectCreateParams as ProjectCreateParams
from .project_delete_params import ProjectDeleteParams as ProjectDeleteParams
from .project_list_response import ProjectListResponse as ProjectListResponse
from .deployment_list_params import DeploymentListParams as DeploymentListParams
from .project_list_env_params import ProjectListEnvParams as ProjectListEnvParams
from .deployment_cancel_params import DeploymentCancelParams as DeploymentCancelParams
from .deployment_create_params import DeploymentCreateParams as DeploymentCreateParams
from .deployment_stream_params import DeploymentStreamParams as DeploymentStreamParams
from .project_create_env_params import ProjectCreateEnvParams as ProjectCreateEnvParams
from .deployment_get_logs_params import DeploymentGetLogsParams as DeploymentGetLogsParams
from .deployment_retrieve_params import DeploymentRetrieveParams as DeploymentRetrieveParams
from .workflow_get_status_params import WorkflowGetStatusParams as WorkflowGetStatusParams
from .project_create_domain_params import ProjectCreateDomainParams as ProjectCreateDomainParams
from .project_get_runtime_logs_params import ProjectGetRuntimeLogsParams as ProjectGetRuntimeLogsParams
from .project_list_deployments_params import ProjectListDeploymentsParams as ProjectListDeploymentsParams
from .project_create_deployment_params import ProjectCreateDeploymentParams as ProjectCreateDeploymentParams



"""
Live API smoke tests for the CaseDev Python SDK.

These tests hit a real API server (staging or production) to verify
the SDK works end-to-end. They are intentionally read-only — no
resources are created, mutated, or deleted.

This file is custom code that persists across Stainless SDK regeneration
since Stainless never touches files it didn't generate.

Configuration (via environment variables):
  CASEDEV_API_KEY       - Required. A valid API key for the target environment.
  CASEDEV_BASE_URL      - Optional. Defaults to https://api.case.dev.

Usage:
  pytest tests/custom/ -m smoke
  pytest tests/custom/ -m smoke --timeout=30
"""

from __future__ import annotations

import os

import pytest

from casedev import Casedev

# ---------------------------------------------------------------------------
# Skip the entire module if no API key is configured.
# This lets the test suite run cleanly in environments that only have
# Prism (mock) credentials.
# ---------------------------------------------------------------------------
pytestmark = pytest.mark.smoke

LIVE_API_KEY = os.environ.get("CASEDEV_API_KEY")

skip_no_api_key = pytest.mark.skipif(
    not LIVE_API_KEY,
    reason="CASEDEV_API_KEY not set — live smoke tests require a real API key",
)


@pytest.fixture(scope="module")
def live_client() -> Casedev:
    """Create a SDK client pointed at the live (or staging) API."""
    return Casedev(
        # CASEDEV_BASE_URL is picked up automatically by the client if set.
        # If unset, the client defaults to https://api.case.dev (production).
    )


# ---------------------------------------------------------------------------
# Smoke tests — read-only, no side effects
# ---------------------------------------------------------------------------


@skip_no_api_key
class TestLlmSmoke:
    """Validate the /llm/config endpoint returns a well-formed model list."""

    def test_get_config_returns_models(self, live_client: Casedev) -> None:
        config = live_client.llm.get_config()
        assert config.models is not None, "Expected models list, got None"
        assert len(config.models) > 0, "Expected at least one model in config"

    def test_get_config_model_has_required_fields(self, live_client: Casedev) -> None:
        config = live_client.llm.get_config()
        model = config.models[0]
        assert model.id, "Model missing 'id'"
        assert model.name, "Model missing 'name'"
        assert model.api_model_type, "Model missing 'modelType'"

    def test_get_config_raw_response(self, live_client: Casedev) -> None:
        """Verify the raw HTTP response is well-formed."""
        response = live_client.llm.with_raw_response.get_config()
        assert response.status_code == 200
        assert "application/json" in response.headers.get("content-type", "")


@skip_no_api_key
class TestVaultSmoke:
    """Validate the /vault list endpoint returns a well-formed response."""

    def test_vault_list_returns_response(self, live_client: Casedev) -> None:
        result = live_client.vault.list()
        # total can be 0 for a fresh org, but the field should exist
        assert result.total is not None, "Expected 'total' field in vault list response"
        assert result.vaults is not None, "Expected 'vaults' list in response"

    def test_vault_list_raw_response(self, live_client: Casedev) -> None:
        """Verify the raw HTTP response is well-formed."""
        response = live_client.vault.with_raw_response.list()
        assert response.status_code == 200
        assert "application/json" in response.headers.get("content-type", "")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["V1GetCitationsResponse", "Citation", "CitationComponents", "CitationSpan"]


class CitationComponents(BaseModel):
    """Structured Bluebook components. Null if citation format is not recognized."""

    case_name: Optional[str] = FieldInfo(alias="caseName", default=None)
    """Case name, e.g., "Bush v. Gore" """

    court: Optional[str] = None
    """Court identifier"""

    page: Optional[int] = None
    """Starting page number"""

    pin_cite: Optional[int] = FieldInfo(alias="pinCite", default=None)
    """Pin cite (specific page)"""

    reporter: Optional[str] = None
    """Reporter abbreviation, e.g., "U.S." """

    volume: Optional[int] = None
    """Volume number"""

    year: Optional[int] = None
    """Decision year"""


class CitationSpan(BaseModel):
    end: Optional[int] = None

    start: Optional[int] = None


class Citation(BaseModel):
    components: Optional[CitationComponents] = None
    """Structured Bluebook components. Null if citation format is not recognized."""

    found: Optional[bool] = None
    """Whether citation was found in CourtListener database"""

    normalized: Optional[str] = None
    """Normalized citation string"""

    original: Optional[str] = None
    """Original citation as found in text"""

    span: Optional[CitationSpan] = None


class V1GetCitationsResponse(BaseModel):
    citations: Optional[List[Citation]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import date

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["V1PatentSearchResponse", "Result"]


class Result(BaseModel):
    application_number: Optional[str] = FieldInfo(alias="applicationNumber", default=None)
    """Patent application serial number"""

    application_type: Optional[str] = FieldInfo(alias="applicationType", default=None)
    """Application type (Utility, Design, Plant, etc.)"""

    assignees: Optional[List[str]] = None
    """List of assignee/owner names"""

    entity_status: Optional[str] = FieldInfo(alias="entityStatus", default=None)
    """Entity status (e.g. "Small Entity", "Micro Entity")"""

    filing_date: Optional[date] = FieldInfo(alias="filingDate", default=None)
    """Date the application was filed"""

    grant_date: Optional[date] = FieldInfo(alias="grantDate", default=None)
    """Date the patent was granted"""

    inventors: Optional[List[str]] = None
    """List of inventor names"""

    patent_number: Optional[str] = FieldInfo(alias="patentNumber", default=None)
    """Granted patent number (if granted)"""

    status: Optional[str] = None
    """Current application status (e.g. "Patented Case", "Pending")"""

    title: Optional[str] = None
    """Invention title"""


class V1PatentSearchResponse(BaseModel):
    limit: Optional[int] = None
    """Number of results returned"""

    offset: Optional[int] = None
    """Current pagination offset"""

    query: Optional[str] = None
    """Original search query"""

    results: Optional[List[Result]] = None
    """Array of matching patent applications"""

    total_results: Optional[int] = FieldInfo(alias="totalResults", default=None)
    """Total number of matching patent applications"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["ExecuteCreateParams", "Sandbox"]


class ExecuteCreateParams(TypedDict, total=False):
    prompt: Required[str]
    """Task prompt for the agent"""

    disabled_tools: Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="disabledTools")]
    """Denylist of tools the agent cannot use"""

    enabled_tools: Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="enabledTools")]
    """Allowlist of tools the agent can use"""

    guidance: Optional[str]
    """Additional context or constraints for this run"""

    instructions: str
    """System instructions.

    Defaults to a general-purpose legal assistant prompt if not provided.
    """

    model: str
    """LLM model identifier. Defaults to anthropic/claude-sonnet-4.6"""

    object_ids: Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="objectIds")]
    """Scope this run to specific vault object IDs.

    The agent will only access these objects.
    """

    sandbox: Optional[Sandbox]
    """Custom sandbox resources (cpu, memoryMiB)"""

    vault_ids: Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="vaultIds")]
    """Restrict agent to specific vault IDs"""


class Sandbox(TypedDict, total=False):
    """Custom sandbox resources (cpu, memoryMiB)"""

    cpu: int
    """Number of CPUs"""

    memory_mi_b: Annotated[int, PropertyInfo(alias="memoryMiB")]
    """Memory in MiB"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .v1 import (
    V1Resource,
    AsyncV1Resource,
    V1ResourceWithRawResponse,
    AsyncV1ResourceWithRawResponse,
    V1ResourceWithStreamingResponse,
    AsyncV1ResourceWithStreamingResponse,
)
from .projects import (
    ProjectsResource,
    AsyncProjectsResource,
    ProjectsResourceWithRawResponse,
    AsyncProjectsResourceWithRawResponse,
    ProjectsResourceWithStreamingResponse,
    AsyncProjectsResourceWithStreamingResponse,
)
from .workflows import (
    WorkflowsResource,
    AsyncWorkflowsResource,
    WorkflowsResourceWithRawResponse,
    AsyncWorkflowsResourceWithRawResponse,
    WorkflowsResourceWithStreamingResponse,
    AsyncWorkflowsResourceWithStreamingResponse,
)
from .deployments import (
    DeploymentsResource,
    AsyncDeploymentsResource,
    DeploymentsResourceWithRawResponse,
    AsyncDeploymentsResourceWithRawResponse,
    DeploymentsResourceWithStreamingResponse,
    AsyncDeploymentsResourceWithStreamingResponse,
)

__all__ = [
    "DeploymentsResource",
    "AsyncDeploymentsResource",
    "DeploymentsResourceWithRawResponse",
    "AsyncDeploymentsResourceWithRawResponse",
    "DeploymentsResourceWithStreamingResponse",
    "AsyncDeploymentsResourceWithStreamingResponse",
    "ProjectsResource",
    "AsyncProjectsResource",
    "ProjectsResourceWithRawResponse",
    "AsyncProjectsResourceWithRawResponse",
    "ProjectsResourceWithStreamingResponse",
    "AsyncProjectsResourceWithStreamingResponse",
    "WorkflowsResource",
    "AsyncWorkflowsResource",
    "WorkflowsResourceWithRawResponse",
    "AsyncWorkflowsResourceWithRawResponse",
    "WorkflowsResourceWithStreamingResponse",
    "AsyncWorkflowsResourceWithStreamingResponse",
    "V1Resource",
    "AsyncV1Resource",
    "V1ResourceWithRawResponse",
    "AsyncV1ResourceWithRawResponse",
    "V1ResourceWithStreamingResponse",
    "AsyncV1ResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ProjectCreateDomainParams"]


class ProjectCreateDomainParams(TypedDict, total=False):
    domain: Required[str]
    """Domain name to add"""

    git_branch: Annotated[str, PropertyInfo(alias="gitBranch")]
    """Git branch to associate with this domain"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["V1ResearchParams"]


class V1ResearchParams(TypedDict, total=False):
    instructions: Required[str]
    """Research instructions or query"""

    model: Literal["fast", "normal", "pro"]
    """Research quality level - fast (quick), normal (balanced), pro (comprehensive)"""

    output_schema: Annotated[object, PropertyInfo(alias="outputSchema")]
    """Optional JSON schema to structure the research output"""

    query: str
    """Alias for instructions (for convenience)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from casedev import Casedev, AsyncCasedev
from tests.utils import assert_matches_type
from casedev.types.agent.v1 import (
    AgentListResponse,
    AgentCreateResponse,
    AgentDeleteResponse,
    AgentUpdateResponse,
    AgentRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAgents:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Casedev) -> None:
        agent = client.agent.v1.agents.create(
            instructions="instructions",
            name="name",
        )
        assert_matches_type(AgentCreateResponse, agent, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Casedev) -> None:
        agent = client.agent.v1.agents.create(
            instructions="instructions",
            name="name",
            description="description",
            disabled_tools=["string"],
            enabled_tools=["string"],
            model="model",
            sandbox={
                "cpu": 0,
                "memory_mi_b": 0,
            },
            vault_groups=["string"],
            vault_ids=["string"],
        )
        assert_matches_type(AgentCreateResponse, agent, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Casedev) -> None:
        response = client.agent.v1.agents.with_raw_response.create(
            instructions="instructions",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentCreateResponse, agent, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Casedev) -> None:
        with client.agent.v1.agents.with_streaming_response.create(
            instructions="instructions",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentCreateResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Casedev) -> None:
        agent = client.agent.v1.agents.retrieve(
            "id",
        )
        assert_matches_type(AgentRetrieveResponse, agent, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Casedev) -> None:
        response = client.agent.v1.agents.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentRetrieveResponse, agent, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Casedev) -> None:
        with client.agent.v1.agents.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentRetrieveResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v1.agents.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Casedev) -> None:
        agent = client.agent.v1.agents.update(
            id="id",
        )
        assert_matches_type(AgentUpdateResponse, agent, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Casedev) -> None:
        agent = client.agent.v1.agents.update(
            id="id",
            description="description",
            disabled_tools=["string"],
            enabled_tools=["string"],
            instructions="instructions",
            model="model",
            name="name",
            sandbox={},
            vault_groups=["string"],
            vault_ids=["string"],
        )
        assert_matches_type(AgentUpdateResponse, agent, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Casedev) -> None:
        response = client.agent.v1.agents.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentUpdateResponse, agent, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Casedev) -> None:
        with client.agent.v1.agents.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentUpdateResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v1.agents.with_raw_response.update(
                id="",
            )

    @parametrize
    def test_method_list(self, client: Casedev) -> None:
        agent = client.agent.v1.agents.list()
        assert_matches_type(AgentListResponse, agent, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Casedev) -> None:
        response = client.agent.v1.agents.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentListResponse, agent, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Casedev) -> None:
        with client.agent.v1.agents.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentListResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Casedev) -> None:
        agent = client.agent.v1.agents.delete(
            "id",
        )
        assert_matches_type(AgentDeleteResponse, agent, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Casedev) -> None:
        response = client.agent.v1.agents.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentDeleteResponse, agent, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Casedev) -> None:
        with client.agent.v1.agents.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentDeleteResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v1.agents.with_raw_response.delete(
                "",
            )


class TestAsyncAgents:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncCasedev) -> None:
        agent = await async_client.agent.v1.agents.create(
            instructions="instructions",
            name="name",
        )
        assert_matches_type(AgentCreateResponse, agent, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncCasedev) -> None:
        agent = await async_client.agent.v1.agents.create(
            instructions="instructions",
            name="name",
            description="description",
            disabled_tools=["string"],
            enabled_tools=["string"],
            model="model",
            sandbox={
                "cpu": 0,
                "memory_mi_b": 0,
            },
            vault_groups=["string"],
            vault_ids=["string"],
        )
        assert_matches_type(AgentCreateResponse, agent, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.agents.with_raw_response.create(
            instructions="instructions",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentCreateResponse, agent, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.agents.with_streaming_response.create(
            instructions="instructions",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentCreateResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncCasedev) -> None:
        agent = await async_client.agent.v1.agents.retrieve(
            "id",
        )
        assert_matches_type(AgentRetrieveResponse, agent, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.agents.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentRetrieveResponse, agent, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.agents.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentRetrieveResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v1.agents.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncCasedev) -> None:
        agent = await async_client.agent.v1.agents.update(
            id="id",
        )
        assert_matches_type(AgentUpdateResponse, agent, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncCasedev) -> None:
        agent = await async_client.agent.v1.agents.update(
            id="id",
            description="description",
            disabled_tools=["string"],
            enabled_tools=["string"],
            instructions="instructions",
            model="model",
            name="name",
            sandbox={},
            vault_groups=["string"],
            vault_ids=["string"],
        )
        assert_matches_type(AgentUpdateResponse, agent, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.agents.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentUpdateResponse, agent, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.agents.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentUpdateResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v1.agents.with_raw_response.update(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncCasedev) -> None:
        agent = await async_client.agent.v1.agents.list()
        assert_matches_type(AgentListResponse, agent, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.agents.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentListResponse, agent, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.agents.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentListResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncCasedev) -> None:
        agent = await async_client.agent.v1.agents.delete(
            "id",
        )
        assert_matches_type(AgentDeleteResponse, agent, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.agents.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentDeleteResponse, agent, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.agents.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentDeleteResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v1.agents.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ObjectDeleteParams"]


class ObjectDeleteParams(TypedDict, total=False):
    id: Required[str]

    force: Literal["true"]
    """Force delete a stuck document that is still in 'processing' state.

    Use this if a document got stuck during ingestion (e.g., OCR timeout).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_custom_raw_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.voice.v1 import speak_create_params

__all__ = ["SpeakResource", "AsyncSpeakResource"]


class SpeakResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpeakResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return SpeakResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeakResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return SpeakResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        text: str,
        apply_text_normalization: bool | Omit = omit,
        enable_logging: bool | Omit = omit,
        language_code: str | Omit = omit,
        model_id: Literal["eleven_multilingual_v2", "eleven_turbo_v2", "eleven_monolingual_v1"] | Omit = omit,
        next_text: str | Omit = omit,
        optimize_streaming_latency: int | Omit = omit,
        output_format: Literal["mp3_44100_128", "mp3_44100_192", "pcm_16000", "pcm_22050", "pcm_24000", "pcm_44100"]
        | Omit = omit,
        previous_text: str | Omit = omit,
        seed: int | Omit = omit,
        voice_id: str | Omit = omit,
        voice_settings: speak_create_params.VoiceSettings | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """Convert text to natural-sounding audio using ElevenLabs voices.

        Ideal for
        creating audio summaries of legal documents, client presentations, or
        accessibility features. Supports multiple languages and voice customization.

        Args:
          text: Text to convert to speech

          apply_text_normalization: Apply automatic text normalization

          enable_logging: Enable request logging

          language_code: Language code for multilingual models

          model_id: ElevenLabs model ID

          next_text: Next context for better pronunciation

          optimize_streaming_latency: Optimize for streaming latency (0-4)

          output_format: Audio output format

          previous_text: Previous context for better pronunciation

          seed: Seed for reproducible generation

          voice_id: ElevenLabs voice ID (defaults to Rachel - professional, clear)

          voice_settings: Voice customization settings

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "audio/mpeg", **(extra_headers or {})}
        return self._post(
            "/voice/v1/speak",
            body=maybe_transform(
                {
                    "text": text,
                    "apply_text_normalization": apply_text_normalization,
                    "enable_logging": enable_logging,
                    "language_code": language_code,
                    "model_id": model_id,
                    "next_text": next_text,
                    "optimize_streaming_latency": optimize_streaming_latency,
                    "output_format": output_format,
                    "previous_text": previous_text,
                    "seed": seed,
                    "voice_id": voice_id,
                    "voice_settings": voice_settings,
                },
                speak_create_params.SpeakCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )


class AsyncSpeakResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpeakResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeakResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeakResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return AsyncSpeakResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        text: str,
        apply_text_normalization: bool | Omit = omit,
        enable_logging: bool | Omit = omit,
        language_code: str | Omit = omit,
        model_id: Literal["eleven_multilingual_v2", "eleven_turbo_v2", "eleven_monolingual_v1"] | Omit = omit,
        next_text: str | Omit = omit,
        optimize_streaming_latency: int | Omit = omit,
        output_format: Literal["mp3_44100_128", "mp3_44100_192", "pcm_16000", "pcm_22050", "pcm_24000", "pcm_44100"]
        | Omit = omit,
        previous_text: str | Omit = omit,
        seed: int | Omit = omit,
        voice_id: str | Omit = omit,
        voice_settings: speak_create_params.VoiceSettings | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """Convert text to natural-sounding audio using ElevenLabs voices.

        Ideal for
        creating audio summaries of legal documents, client presentations, or
        accessibility features. Supports multiple languages and voice customization.

        Args:
          text: Text to convert to speech

          apply_text_normalization: Apply automatic text normalization

          enable_logging: Enable request logging

          language_code: Language code for multilingual models

          model_id: ElevenLabs model ID

          next_text: Next context for better pronunciation

          optimize_streaming_latency: Optimize for streaming latency (0-4)

          output_format: Audio output format

          previous_text: Previous context for better pronunciation

          seed: Seed for reproducible generation

          voice_id: ElevenLabs voice ID (defaults to Rachel - professional, clear)

          voice_settings: Voice customization settings

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "audio/mpeg", **(extra_headers or {})}
        return await self._post(
            "/voice/v1/speak",
            body=await async_maybe_transform(
                {
                    "text": text,
                    "apply_text_normalization": apply_text_normalization,
                    "enable_logging": enable_logging,
                    "language_code": language_code,
                    "model_id": model_id,
                    "next_text": next_text,
                    "optimize_streaming_latency": optimize_streaming_latency,
                    "output_format": output_format,
                    "previous_text": previous_text,
                    "seed": seed,
                    "voice_id": voice_id,
                    "voice_settings": voice_settings,
                },
                speak_create_params.SpeakCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )


class SpeakResourceWithRawResponse:
    def __init__(self, speak: SpeakResource) -> None:
        self._speak = speak

        self.create = to_custom_raw_response_wrapper(
            speak.create,
            BinaryAPIResponse,
        )


class AsyncSpeakResourceWithRawResponse:
    def __init__(self, speak: AsyncSpeakResource) -> None:
        self._speak = speak

        self.create = async_to_custom_raw_response_wrapper(
            speak.create,
            AsyncBinaryAPIResponse,
        )


class SpeakResourceWithStreamingResponse:
    def __init__(self, speak: SpeakResource) -> None:
        self._speak = speak

        self.create = to_custom_streamed_response_wrapper(
            speak.create,
            StreamedBinaryAPIResponse,
        )


class AsyncSpeakResourceWithStreamingResponse:
    def __init__(self, speak: AsyncSpeakResource) -> None:
        self._speak = speak

        self.create = async_to_custom_streamed_response_wrapper(
            speak.create,
            AsyncStreamedBinaryAPIResponse,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ObjectGetSummarizeJobResponse"]


class ObjectGetSummarizeJobResponse(BaseModel):
    completed_at: Optional[datetime] = FieldInfo(alias="completedAt", default=None)
    """When the job completed"""

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)
    """When the job was created"""

    error: Optional[str] = None
    """Error message (if failed)"""

    job_id: Optional[str] = FieldInfo(alias="jobId", default=None)
    """Case.dev job ID"""

    result_filename: Optional[str] = FieldInfo(alias="resultFilename", default=None)
    """Filename of the result document (if completed)"""

    result_object_id: Optional[str] = FieldInfo(alias="resultObjectId", default=None)
    """ID of the result document (if completed)"""

    source_object_id: Optional[str] = FieldInfo(alias="sourceObjectId", default=None)
    """ID of the source document"""

    status: Optional[Literal["pending", "processing", "completed", "failed"]] = None
    """Current job status"""

    workflow_type: Optional[str] = FieldInfo(alias="workflowType", default=None)
    """Type of workflow being executed"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["V1SimilarResponse", "SimilarSource"]


class SimilarSource(BaseModel):
    published_date: Optional[str] = FieldInfo(alias="publishedDate", default=None)
    """Publication date"""

    snippet: Optional[str] = None
    """Text excerpt from the document"""

    source: Optional[str] = None
    """Domain of the source"""

    title: Optional[str] = None
    """Title of the document"""

    url: Optional[str] = None
    """URL of the similar source"""


class V1SimilarResponse(BaseModel):
    found: Optional[int] = None
    """Number of similar sources found"""

    hint: Optional[str] = None
    """Usage guidance"""

    jurisdiction: Optional[str] = None
    """Jurisdiction filter applied"""

    similar_sources: Optional[List[SimilarSource]] = FieldInfo(alias="similarSources", default=None)

    source_url: Optional[str] = FieldInfo(alias="sourceUrl", default=None)
    """Original source URL"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.llm.v1 import chat_create_completion_params
from ....types.llm.v1.chat_create_completion_response import ChatCreateCompletionResponse

__all__ = ["ChatResource", "AsyncChatResource"]


class ChatResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return ChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return ChatResourceWithStreamingResponse(self)

    def create_completion(
        self,
        *,
        messages: Iterable[chat_create_completion_params.Message],
        casemark_show_reasoning: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        max_tokens: int | Omit = omit,
        model: str | Omit = omit,
        presence_penalty: float | Omit = omit,
        stream: bool | Omit = omit,
        temperature: float | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatCreateCompletionResponse:
        """Create a completion for the provided prompt and parameters.

        Compatible with
        OpenAI's chat completions API. Supports 40+ models including GPT-4, Claude,
        Gemini, and CaseMark legal AI models. Includes streaming support, token
        counting, and usage tracking.

        Args:
          messages: List of messages comprising the conversation

          casemark_show_reasoning: CaseMark-only: when true, allows reasoning fields in responses. Defaults to
              false (reasoning is suppressed).

          frequency_penalty: Frequency penalty parameter

          max_tokens: Maximum number of tokens to generate

          model: Model to use for completion. Defaults to casemark/casemark-core-3 if not
              specified

          presence_penalty: Presence penalty parameter

          stream: Whether to stream back partial progress

          temperature: Sampling temperature between 0 and 2

          top_p: Nucleus sampling parameter

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/llm/v1/chat/completions",
            body=maybe_transform(
                {
                    "messages": messages,
                    "casemark_show_reasoning": casemark_show_reasoning,
                    "frequency_penalty": frequency_penalty,
                    "max_tokens": max_tokens,
                    "model": model,
                    "presence_penalty": presence_penalty,
                    "stream": stream,
                    "temperature": temperature,
                    "top_p": top_p,
                },
                chat_create_completion_params.ChatCreateCompletionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCreateCompletionResponse,
        )


class AsyncChatResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return AsyncChatResourceWithStreamingResponse(self)

    async def create_completion(
        self,
        *,
        messages: Iterable[chat_create_completion_params.Message],
        casemark_show_reasoning: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        max_tokens: int | Omit = omit,
        model: str | Omit = omit,
        presence_penalty: float | Omit = omit,
        stream: bool | Omit = omit,
        temperature: float | Omit = omit,
        top_p: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatCreateCompletionResponse:
        """Create a completion for the provided prompt and parameters.

        Compatible with
        OpenAI's chat completions API. Supports 40+ models including GPT-4, Claude,
        Gemini, and CaseMark legal AI models. Includes streaming support, token
        counting, and usage tracking.

        Args:
          messages: List of messages comprising the conversation

          casemark_show_reasoning: CaseMark-only: when true, allows reasoning fields in responses. Defaults to
              false (reasoning is suppressed).

          frequency_penalty: Frequency penalty parameter

          max_tokens: Maximum number of tokens to generate

          model: Model to use for completion. Defaults to casemark/casemark-core-3 if not
              specified

          presence_penalty: Presence penalty parameter

          stream: Whether to stream back partial progress

          temperature: Sampling temperature between 0 and 2

          top_p: Nucleus sampling parameter

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/llm/v1/chat/completions",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "casemark_show_reasoning": casemark_show_reasoning,
                    "frequency_penalty": frequency_penalty,
                    "max_tokens": max_tokens,
                    "model": model,
                    "presence_penalty": presence_penalty,
                    "stream": stream,
                    "temperature": temperature,
                    "top_p": top_p,
                },
                chat_create_completion_params.ChatCreateCompletionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCreateCompletionResponse,
        )


class ChatResourceWithRawResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

        self.create_completion = to_raw_response_wrapper(
            chat.create_completion,
        )


class AsyncChatResourceWithRawResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

        self.create_completion = async_to_raw_response_wrapper(
            chat.create_completion,
        )


class ChatResourceWithStreamingResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

        self.create_completion = to_streamed_response_wrapper(
            chat.create_completion,
        )


class AsyncChatResourceWithStreamingResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

        self.create_completion = async_to_streamed_response_wrapper(
            chat.create_completion,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .v1_get_usage_params import V1GetUsageParams as V1GetUsageParams
from .v1_get_usage_response import V1GetUsageResponse as V1GetUsageResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["V1GetFullTextResponse"]


class V1GetFullTextResponse(BaseModel):
    author: Optional[str] = None
    """Author or court"""

    character_count: Optional[int] = FieldInfo(alias="characterCount", default=None)
    """Total characters in text"""

    highlights: Optional[List[str]] = None
    """Highlighted relevant passages"""

    published_date: Optional[str] = FieldInfo(alias="publishedDate", default=None)
    """Publication date"""

    summary: Optional[str] = None
    """AI-generated summary"""

    text: Optional[str] = None
    """Full document text"""

    title: Optional[str] = None
    """Document title"""

    url: Optional[str] = None
    """Document URL"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options

__all__ = ["GroupsResource", "AsyncGroupsResource"]


class GroupsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> GroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return GroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return GroupsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Create vault group"""
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/vault/groups",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def update(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Update vault group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            f"/vault/groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """List vault groups"""
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            "/vault/groups",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def delete(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete vault group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/vault/groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncGroupsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return AsyncGroupsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Create vault group"""
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/vault/groups",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def update(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Update vault group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            f"/vault/groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """List vault groups"""
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            "/vault/groups",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def delete(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete vault group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/vault/groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class GroupsResourceWithRawResponse:
    def __init__(self, groups: GroupsResource) -> None:
        self._groups = groups

        self.create = to_raw_response_wrapper(
            groups.create,
        )
        self.update = to_raw_response_wrapper(
            groups.update,
        )
        self.list = to_raw_response_wrapper(
            groups.list,
        )
        self.delete = to_raw_response_wrapper(
            groups.delete,
        )


class AsyncGroupsResourceWithRawResponse:
    def __init__(self, groups: AsyncGroupsResource) -> None:
        self._groups = groups

        self.create = async_to_raw_response_wrapper(
            groups.create,
        )
        self.update = async_to_raw_response_wrapper(
            groups.update,
        )
        self.list = async_to_raw_response_wrapper(
            groups.list,
        )
        self.delete = async_to_raw_response_wrapper(
            groups.delete,
        )


class GroupsResourceWithStreamingResponse:
    def __init__(self, groups: GroupsResource) -> None:
        self._groups = groups

        self.create = to_streamed_response_wrapper(
            groups.create,
        )
        self.update = to_streamed_response_wrapper(
            groups.update,
        )
        self.list = to_streamed_response_wrapper(
            groups.list,
        )
        self.delete = to_streamed_response_wrapper(
            groups.delete,
        )


class AsyncGroupsResourceWithStreamingResponse:
    def __init__(self, groups: AsyncGroupsResource) -> None:
        self._groups = groups

        self.create = async_to_streamed_response_wrapper(
            groups.create,
        )
        self.update = async_to_streamed_response_wrapper(
            groups.update,
        )
        self.list = async_to_streamed_response_wrapper(
            groups.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            groups.delete,
        )
